diff -aru vlc-1.1.0.orig/include/vlc/libvlc_media_player.h vlc-1.1/include/vlc/libvlc_media_player.h
--- vlc-1.1.0.orig/include/vlc/libvlc_media_player.h	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/include/vlc/libvlc_media_player.h	2010-06-23 16:48:39.036859508 +0200
@@ -378,6 +378,19 @@
 VLC_PUBLIC_API int libvlc_media_player_will_play( libvlc_media_player_t *p_mi );
 
 /**
+ * Current (fill) size of decoder buffer
+ *
+ * \param p_mi the Media Player
+ * \param pi_count pointer to int to receive count of buffer items or NULL
+ * \param pi_size pointer to size_t to receive size of buffer in bytes or NULL
+ * \param pi_length pointer to libvlc_time_t to receive bufferlength in ms or NULL
+ */
+VLC_PUBLIC_API void libvlc_media_player_get_buffer_state( libvlc_media_player_t *p_mi,
+                                                          int *pi_count,
+                                                          size_t *pi_size,
+                                                          libvlc_time_t *pi_length );
+
+/**
  * Get title chapter count
  *
  * \param p_mi the Media Player
diff -aru vlc-1.1.0.orig/include/vlc_input.h vlc-1.1/include/vlc_input.h
--- vlc-1.1.0.orig/include/vlc_input.h	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/include/vlc_input.h	2010-06-22 17:52:36.534186416 +0200
@@ -518,6 +518,9 @@
     /* External clock managments */
     INPUT_GET_PCR_SYSTEM,   /* arg1=mtime_t *, arg2=mtime_t *       res=can fail */
     INPUT_MODIFY_PCR_SYSTEM,/* arg1=int absolute, arg2=mtime_t      res=can fail */
+
+    /* Input Decoder */
+    INPUT_GET_DECODER_FIFO_STATE, /* arg1=int*, arg2=size_t*, arg3=mtime_t*     res=cannot fail */
 };
 
 /** @}*/
diff -aru vlc-1.1.0.orig/src/control/media_player.c vlc-1.1/src/control/media_player.c
--- vlc-1.1.0.orig/src/control/media_player.c	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/control/media_player.c	2010-06-23 16:48:20.521934916 +0200
@@ -1094,6 +1100,27 @@
     return b_will_play;
 }
 
+void libvlc_media_player_get_buffer_state( libvlc_media_player_t *p_mi, int *pi_count, size_t *pi_size, libvlc_time_t *pi_length )
+{
+    input_thread_t *p_input_thread = libvlc_get_input_thread ( p_mi );
+    int            i_count = 0;
+    size_t         i_size = 0;
+    mtime_t        i_length = 0;
+
+    if( p_input_thread )
+    {
+        input_Control( p_input_thread, INPUT_GET_DECODER_FIFO_STATE, &i_count, &i_size, &i_length );
+        vlc_object_release( p_input_thread );
+    }
+
+    if( pi_count )
+        *pi_count = i_count;
+    if( pi_size )
+        *pi_size = i_size;
+    if( pi_length )
+        *pi_length = i_length / 1000;
+}
+
 int libvlc_media_player_set_rate( libvlc_media_player_t *p_mi, float rate )
 {
     if (rate < 0.)
diff -aru vlc-1.1.0.orig/src/input/control.c vlc-1.1/src/input/control.c
--- vlc-1.1.0.orig/src/input/control.c	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/input/control.c	2010-06-22 17:51:32.055432596 +0200
@@ -349,6 +349,15 @@
             vlc_mutex_unlock( &p_input->p->p_item->lock );
             return VLC_SUCCESS;
 
+        case INPUT_GET_DECODER_FIFO_STATE:
+	{
+            int *pi_count = (int*)va_arg( args, int * );
+            size_t *pi_size = (size_t*)va_arg( args, size_t * );
+            mtime_t *pi_length = (mtime_t*)va_arg( args, mtime_t * );
+
+            return es_out_Control( p_input->p->p_es_out_display, ES_OUT_GET_DECODER_FIFO_STATE, pi_count, pi_size, pi_length );
+	}
+
         case INPUT_ADD_SLAVE:
             psz = (char*)va_arg( args, char * );
             if( psz && *psz )
diff -aru vlc-1.1.0.orig/src/input/decoder.c vlc-1.1/src/input/decoder.c
--- vlc-1.1.0.orig/src/input/decoder.c	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/input/decoder.c	2010-06-22 18:06:18.343479720 +0200
@@ -624,6 +624,27 @@
     return block_FifoSize( p_owner->p_fifo );
 }
 
+void input_DecoderGetFifoState( decoder_t *p_dec, int *pi_count, size_t *pi_size, mtime_t *pi_length )
+{
+    decoder_owner_sys_t *p_owner = p_dec->p_owner;
+
+    vlc_mutex_lock( &p_owner->lock );
+
+    /* avoid locking */
+    if(block_FifoSize( p_owner->p_fifo ) > 0) {
+        block_ChainProperties( block_FifoShow( p_owner->p_fifo ), pi_count, pi_size, pi_length );
+    } else {
+        if( pi_count )
+            *pi_count = 0;
+        if( pi_size )
+            *pi_size = 0;
+        if( pi_length )
+            *pi_length = 0;
+    }
+
+    vlc_mutex_unlock( &p_owner->lock );
+}
+
 void input_DecoderGetObjects( decoder_t *p_dec,
                               vout_thread_t **pp_vout, aout_instance_t **pp_aout )
 {
diff -aru vlc-1.1.0.orig/src/input/decoder.h vlc-1.1/src/input/decoder.h
--- vlc-1.1.0.orig/src/input/decoder.h	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/input/decoder.h	2010-06-22 17:31:06.479617327 +0200
@@ -105,6 +105,11 @@
 size_t input_DecoderGetFifoSize( decoder_t *p_dec );
 
 /**
+ * This function returns some information on the decoder fifo
+ */
+void input_DecoderGetFifoState( decoder_t *p_dec, int *pi_count, size_t *pi_size, mtime_t *pi_length );
+
+/**
  * This function returns the objects associated to a decoder
  *
  * They must be released using vlc_object_release().
diff -aru vlc-1.1.0.orig/src/input/es_out.c vlc-1.1/src/input/es_out.c
--- vlc-1.1.0.orig/src/input/es_out.c	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/input/es_out.c	2010-06-22 18:07:25.171748043 +0200
@@ -452,6 +452,45 @@
     return true;
 }
 
+static void EsOutGetDecoderFifoState( es_out_t *out, int *pi_count, size_t *pi_size, mtime_t *pi_length )
+{
+    es_out_sys_t      *p_sys = out->p_sys;
+
+    int               i_count = 0;
+    size_t            i_size = 0;
+    mtime_t           i_length = 0;
+
+    int i;
+
+    for( i = 0; i < p_sys->i_es; i++ )
+    {
+        es_out_id_t *p_es = p_sys->es[i];
+        int         i_thisCount = 0;
+	size_t      i_thisSize = 0;
+	mtime_t     i_thisLength = 0;
+
+
+        if( p_es->p_dec )
+            input_DecoderGetFifoState( p_es->p_dec, &i_thisCount, &i_thisSize, &i_thisLength );
+        if( p_es->p_dec_record )
+            input_DecoderGetFifoState( p_es->p_dec_record, &i_thisCount, &i_thisSize, &i_thisLength );
+
+	i_count += i_thisCount;
+	i_size += i_thisSize;
+
+        /* largest fifo wins */
+	if( i_thisLength > i_length )
+	    i_length = i_thisLength;
+    }
+
+    if( pi_count )
+        *pi_count = i_count;
+    if( pi_size )
+        *pi_size = i_size;
+    if( pi_length )
+        *pi_length = i_length;
+}
+
 static void EsOutSetDelay( es_out_t *out, int i_cat, int64_t i_delay )
 {
     es_out_sys_t *p_sys = out->p_sys;
@@ -2534,6 +2573,15 @@
             return VLC_SUCCESS;
         }
 
+        case ES_OUT_GET_DECODER_FIFO_STATE:
+        {
+            int *pi_count = va_arg( args, int* );
+	    size_t *pi_size = va_arg( args, size_t* );
+	    mtime_t *pi_length = va_arg( args, mtime_t* );
+            EsOutGetDecoderFifoState( out, pi_count, pi_size, pi_length );
+            return VLC_SUCCESS;
+        }
+
         case ES_OUT_SET_DELAY:
         {
             const int i_cat = (int)va_arg( args, int );
diff -aru vlc-1.1.0.orig/src/input/es_out.h vlc-1.1/src/input/es_out.h
--- vlc-1.1.0.orig/src/input/es_out.h	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/input/es_out.h	2010-06-22 17:44:32.293296367 +0200
@@ -80,6 +80,9 @@
 
     /* Set jitter */
     ES_OUT_SET_JITTER,                              /* arg1=mtime_t i_pts_delay arg2= mtime_t i_pts_jitter, arg2=int i_cr_average res=cannot fail */
+
+    /* Get Decoder FIFO state */
+    ES_OUT_GET_DECODER_FIFO_STATE,                  /* arg1=int* pi_count arg2=size_t* pi_size arg3=mtime_t* pi_length res=cannot fail */
 };
 
 static inline void es_out_SetMode( es_out_t *p_out, int i_mode )
@@ -111,6 +114,11 @@
     assert( !i_ret );
     return b;
 }
+static inline void es_out_GetDecoderFifoState( es_out_t *p_out, int *pi_count, size_t *pi_size, mtime_t *pi_length )
+{
+    int i_ret = es_out_Control( p_out, ES_OUT_GET_DECODER_FIFO_STATE, pi_count, pi_size, pi_length );
+    assert( !i_ret );
+}
 static inline void es_out_SetDelay( es_out_t *p_out, int i_cat, mtime_t i_delay )
 {
     int i_ret = es_out_Control( p_out, ES_OUT_SET_DELAY, i_cat, i_delay );
diff -aru vlc-1.1.0.orig/src/input/es_out_timeshift.c vlc-1.1/src/input/es_out_timeshift.c
--- vlc-1.1.0.orig/src/input/es_out_timeshift.c	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/input/es_out_timeshift.c	2010-06-22 18:08:01.523477352 +0200
@@ -484,6 +484,26 @@
 
     return VLC_SUCCESS;
 }
+static int ControlLockedGetDecoderFifoState( es_out_t *p_out, int *pi_count, size_t *pi_size, mtime_t *pi_length )
+{
+    es_out_sys_t *p_sys = p_out->p_sys;
+
+    if( p_sys->b_delayed )
+    {
+        if( pi_count )
+            *pi_count = 0;
+        if( pi_size )
+            *pi_size = 0;
+        if( pi_length )
+            *pi_length = 0;
+    }
+    else
+    {
+        es_out_GetDecoderFifoState( p_sys->p_out, pi_count, pi_size, pi_length );
+    }
+
+    return VLC_SUCCESS;
+}
 static int ControlLockedSetPauseState( es_out_t *p_out, bool b_source_paused, bool b_paused, mtime_t i_date )
 {
     es_out_sys_t *p_sys = p_out->p_sys;
@@ -650,6 +670,13 @@
         bool *pb_buffering = (bool *)va_arg( args, bool* );
         return ControlLockedGetBuffering( p_out, pb_buffering );
     }
+    case ES_OUT_GET_DECODER_FIFO_STATE:
+    {
+        int *pi_count = (int *)va_arg( args, int* );
+        size_t *pi_size = (size_t *)va_arg( args, size_t* );
+	mtime_t *pi_length = (mtime_t *)va_arg( args, mtime_t* );
+        return ControlLockedGetDecoderFifoState( p_out, pi_count, pi_size, pi_length );
+    }
     case ES_OUT_SET_PAUSE_STATE:
     {
         const bool b_source_paused = (bool)va_arg( args, int );
diff -aru vlc-1.1.0.orig/src/libvlc.sym vlc-1.1/src/libvlc.sym
--- vlc-1.1.0.orig/src/libvlc.sym	2010-06-21 21:19:39.000000000 +0200
+++ vlc-1.1.0/src/libvlc.sym	2010-06-23 15:04:10.892074067 +0200
@@ -108,6 +108,7 @@
 libvlc_media_player_next_frame
 libvlc_media_player_event_manager
 libvlc_media_player_get_agl
+libvlc_media_player_get_buffer_state
 libvlc_media_player_get_chapter
 libvlc_media_player_get_chapter_count
 libvlc_media_player_get_chapter_count_for_title
